import numpy as np
from matplotlib import pyplot as plt
# from mayavi import mlab
from mpl_toolkits.mplot3d import Axes3D
import cv2
import math

# gaze_origin_L = [13, 14, 15]
# gaze_origin_R = [16, 17, 18]
# gaze_direct_L = [19, 20, 21]
# gaze_direct_R = [22, 23, 24]
p1 = np.array([1, 1, 1])
p2 = np.array([0, 1, 0])
p3 = np.array([1, 0, 0])

def load_data(path):
    file = open(path, 'r')
    all_lines = file.readlines()
    return all_lines

def norm(v):
    length = len(v)
    end = 0
    for i in range(length):
        end += v[i] * v[i]
    return np.sqrt(end)

def process_data():
    path = "E:\zhangya\Pro\SaccadeVR-mobile-master\SaccadeVR-mobile-master\SaccadeVR_ViveProEye\StandardSaccade_IDnumber.txt"
    all_lines = load_data(path)
    all_keys = all_lines[0].split(',')

    all_gaze_origin_L = []  # save the left eye gaze origin
    all_gaze_origin_R = []  # save the left eye gaze origin
    all_gaze_direct_L = []  # save the left eye gaze direction
    all_gaze_direct_R = []  # save the right eye gaze direction
    # 从眼动数据中提取关键的信息
    for i in range(1, len(all_lines)):
        line = all_lines[i].split(',')
        origin_L = [line[13], line[14], line[15]]
        origin_R = [line[16], line[17], line[18]]
        gaze_L = [line[19], line[20], line[21]]
        gaze_R = [line[22], line[23], line[24]]

        all_gaze_origin_L.append(np.array(origin_L, np.float))
        all_gaze_origin_R.append(np.array(origin_R, np.float))
        all_gaze_direct_L.append(np.array(gaze_L, np.float))
        all_gaze_direct_R.append(np.array(gaze_R, np.float))
    # 对提取的数据进行格式化
    all_gaze_origin_L = np.array(all_gaze_origin_L)
    all_gaze_origin_R = np.array(all_gaze_origin_R)
    all_gaze_direct_L = np.array(all_gaze_direct_L)
    all_gaze_direct_R = np.array(all_gaze_direct_R)

    return all_gaze_origin_R, all_gaze_origin_L, all_gaze_direct_L, all_gaze_direct_R

# 射线list构造
def lights_list(gaze_start, gaze_direct):

    lights = []
    for i in range(len(gaze_start)):
        light = gaze_start[i] + gaze_direct[i]
        lights.append(light)
    return lights

# 判断射线和平面是否相交
def intersection_plane(start, direction):
    global p1, p2, p3
    direction = np.array(direction) / norm(direction)
    # three point build a plane, and compute the intersection which in the plane
    # need to judge the direction of the light,
    plane_norm = np.cross(p2 - p1, p3 - p1)
    t = np.dot((p1 - start), plane_norm) / np.dot(plane_norm, direction)
    # then judge the point is in the triangle or not
    vec = start + t * direction
    return vec, t

# def visual(points):
#     mlab.points3d(points[:, 0], points[:, 1], points[:, 2],
#                   colormap="spectral", scale_factor=0.25)

#     # mlab.points3d(gaze_origin_R[:, 0], gaze_origin_R[:, 1], gaze_origin_R[:, 2], colormap='jet')
#     mlab.show()

#     # fig = plt.figure()
#     # ax = plt.subplot(111, projection="3d")
#     # points = ax.scatter(gaze_origin_L[:, 0], gaze_origin_L[:, 1], gaze_origin_L[:, 2])
#     # plt.show()
#     return

def load_data_v2():
    # the pic in unity is 1024*512
    # path = "E:\\zhangya\\Pro\\textured_mesh_saliency\\fixation\\hlg\\userid1.txt"
    # path = "E:\\zhangya\\Pro\\textured_mesh_saliency\\fixation\\zy\\userid1.txt"
    path = "zy_l/userid4.txt"

    file = open(path)
    lines = file.readlines()
    left_eyes = []
    right_eyes = []
    count = 0
    for line in lines:
        if line[0] == '#':
            continue
        numbers = line.split(',')
        left_position = np.array([numbers[0], numbers[1], numbers[2]], np.float)
        # right_position = np.array([numbers[3], numbers[4], numbers[5]], np.float)
        left_eyes.append(left_position)
        # right_eyes.append(right_position)
        count += 1

    left_eyes = np.array(left_eyes)
    # right_eyes = np.array(right_eyes)

    for i in range(len(left_eyes)):
        left_eyes[i] += np.array([0.5, -0.5, 0])
        # right_eyes[i] += np.array([0.5, -0.5, 0])
        left_eyes[i][1] *= -1
        # right_eyes[i][1] *= -1

    return left_eyes, right_eyes

def valid_p(p):
    if 512 > p >= 0:
        return True
    else:
        return False

def circle_heat(img_gray, x, y, radius):
    for i in range(x - radius, x + radius + 1):
        for j in range(y - radius, y + radius + 1):
            len_x = math.fabs(i - x)
            len_y = math.fabs(j - y)
            dis = math.sqrt(math.pow(len_x, 2) + math.pow(len_y, 2))
            img_gray[i][j] += radius - int(dis)

def pic_resize():
    # imga = cv2.imread("E:\\zhangya\\Pro\\textured_mesh_saliency\\py_2dto3d\\out\\+x0.png")
    # imga = cv2.imread("E:\\zhangya\\Pro\\textured_mesh_saliency\\fixation\\render\\crown_x17.png")
    # imga = cv2.imread("E:\\zhangya\\Pro\\textured_mesh_saliency\\fixation\\render\\killeroo-gold_x17.png")
    imga = cv2.imread("out/-y0.png")
    imga = cv2.resize(imga, (512, 512), 0, 0)
    img = imga

    # img = cv2.flip(img, 1)

    x_dist = 1 / 512
    y_dist = 1 / 512

    left_eyes, right_eyes = load_data_v2()
    index_lists = []

    white = np.array([255, 255, 255], dtype=np.uint8)
    img_gray = np.zeros(512 * 512).reshape([512, 512])

    for i in range(len(left_eyes)):
        left_x = int(left_eyes[i][0] / x_dist)
        left_y = int(left_eyes[i][1] / x_dist)
        # right_x = int(right_eyes[i][0] / y_dist)
        # right_y = int(right_eyes[i][1] / y_dist)

        # if valid_p(left_x) and valid_p(left_y) and valid_p(right_x) and valid_p(right_y):
        if valid_p(left_x) and valid_p(left_y):
            cv2.circle(img, (left_x - 15, left_y), 3, (0, 0, 213), -1)
            # circle_heat(img_gray, left_x, left_y, 5)
            # cv2.circle(img, (right_x, right_y), 3, (213, 0, 213), -1)
            # cv2.circle(img, (int(left_x/2+right_x/2), int(left_y/2+right_y/2)), 3, (0, 213, 0), -1)
        index_lists.append([left_x, left_y])
    
    # img_gray_one = img_gray.reshape(512 * 512)
    # max_heat = max(img_gray_one)
    # min_heat = 0
    # each_d = 1 / max_heat
    # for i in range(512):
    #     for j in range(512):
    #         img[i, j] += img_gray[i][j] * each_d * 255

    # img = cv2.cvtColor(img, cv2.COLOR_RGB2GRAY)
    # img_color = cv2.applyColorMap(img, cv2.COLORMAP_COOL)

    cv2.imshow("wei cai se", img)
    cv2.waitKey(0)


# load_data_v2()
pic_resize()



