import yang.myutil as myutil
from mayavi import mlab
import numpy as np

def load_face_saliency():
    face_saliency = []
    with open("ft_face_saliency.txt", 'r') as file:
        lines = file.readlines()
        for line in lines:
            face_saliency.append(np.float(line))
    return face_saliency

def load_obj(filename):
    vertices_ = []
    faces_ = []
    normals_ = []
    for line in open(filename):
        if len(line) == 0:
            continue
        if line.startswith('#'):
            continue
        values = line.split(' ')
        if values[0] == 'v':
            v = [float(x) for x in values[1:4]]
            v = [v[0], v[1], v[2]]
            vertices_.append(v)
        elif values[0] == 'f':
            f = [int(values[1].split('/')[0])-1,
                 int(values[2].split('/')[0])-1,
                 int(values[3].split('/')[0])-1]
            faces_.append(f)
        elif values[0] == 'vn':
            n = [float(x) for x in values[1:4]]
            n = [n[0], n[1], n[2]]
            normals_.append(n)
    vertices_ = np.array(vertices_)
    faces_ = np.array(faces_)
    normals_ = np.array(normals_)
    return [vertices_, faces_, normals_]

# 使用平均来定义点的Saliency，但是这是符合实际的
def fs_to_vs(face_saliency, vertices, faces):
    v_to_triangles = myutil.get_vertex_to_face(faces)
    areas = []
    for face_i in range(len(faces)):
        area = myutil.area_of_triangle(vertices, faces[face_i])
        areas.append(area)

    # face saliency to vertex saliency based on the area of triangle
    vertex_saliency = np.zeros(len(vertices))
    for i in range(len(vertices)):
        tris = v_to_triangles[i]
        local_areas = []
        for tri_index in tris:
            local_areas.append(areas[tri_index])
            # vertex_saliency[i] += face_saliency[tri_index] * areas[tri_index]
            vertex_saliency[i] += face_saliency[tri_index]

        # vertex_saliency[i] = vertex_saliency[i] / (max(local_areas) - min(local_areas))
        vertex_saliency[i] = vertex_saliency[i] / len(tris)
    return vertex_saliency

# 那么符合实际的人眼感知模型是如何的呢？


vertices, faces, nnorms = load_obj("/home/simula/Dataset/textured_model/bunny/bunny.obj")
face_saliency = load_face_saliency()
vertex_saliency = fs_to_vs(face_saliency, vertices, faces)

myutil.mayavi_with_custom_point(vertices, faces, vertex_saliency)
mlab.show()

