'''
代码的目的
1. 对于6张显著度图，进行超像素划分。
2. 针对超像素划分后的显著度进行显著排序，得到每个超像素的显著度及其显著排序。
3. 根据显著排序进行显著读的融合
4. 得到超像素的显著度
5. 超像素的显著度直接贴回模型，还是在超像素内部进行超点集？
6. 因为我的数据集是带纹理的模型，所以超点集可能不需要吧，可以作为实验对比来写出来。
'''

import cv2
import imageio
import matplotlib.pyplot as plt
import numpy as np

from skimage import color
from skimage.segmentation import slic, mark_boundaries
from mayavi import mlab

import os

def ft_saliency(path):
    img1 = cv2.imread(path)

    img = cv2.cvtColor(img1, cv2.COLOR_BGR2RGB)
    img = cv2.GaussianBlur(img, (5, 5), 0)
    gray_lab = cv2.cvtColor(img, cv2.COLOR_RGB2LAB)

    l_mean = np.mean(gray_lab[:, :, 0])
    a_mean = np.mean(gray_lab[:, :, 1])
    b_mean = np.mean(gray_lab[:, :, 2])
    lab = np.square(gray_lab - np.array([l_mean, a_mean, b_mean]))
    lab = np.sum(lab, axis=2)
    lab = lab / np.max(lab)

    # plt.imshow(lab, cmap='gray')
    # plt.show()
    return lab  # 这不是标准的lab空间的图片
    # so the lab pic could not to be exchanged to rgb
    # the shape of lab is (h,w,)


def load_obj(filename):
    vertices_ = []
    faces_ = []
    for line in open(filename):
        if len(line) == 0:
            continue
        if line.startswith('#'):
            continue
        values = line.split(' ')
        if values[0] == 'v':
            v = [float(x) for x in values[1:4]]
            v = [v[0], v[1], v[2]]
            vertices_.append(v)
        elif values[0] == 'f':
            f = [int(values[1].split('/')[0])-1,
                 int(values[2].split('/')[0])-1,
                 int(values[3].split('/')[0])-1]
            faces_.append(f)
    vertices_ = np.array(vertices_)
    faces_ = np.array(faces_)
    return vertices_, faces_

def compute_SLIC_opencv(path):

    img = cv2.imread(path)
    slic = cv2.ximgproc.createSuperpixelSLIC(img,region_size=20,ruler = 20.0)
    slic.iterate(10)    #迭代次数，越大效果越好
    mask_slic = slic.getLabelContourMask() #获取Mask，超像素边缘Mask==1
    label_slic = slic.getLabels()         #获取超像素标签
    number_slic = slic.getNumberOfSuperpixels()  #获取超像素数目
    mask_inv_slic = cv2.bitwise_not(mask_slic)
    img_slic = cv2.bitwise_and(img, img, mask=mask_inv_slic) #在原图上绘制超像素边界

    return img_slic

def compute_SEEDS_opencv(path):
    img = cv2.imread(path)
    seeds = cv2.ximgproc.createSuperpixelSEEDS(img.shape[1], img.shape[0], img.shape[2], 2000, 15, 3, 5, True)
    seeds.iterate(img, 10)  # 输入图像大小必须与初始化形状相同，迭代次数为10

    mask_seeds = seeds.getLabelContourMask()

    label_seeds = seeds.getLabels()
    h, w = np.array(label_seeds).shape
    for i in range(h):
        for j in range(w):
            label_seeds[i][j] /= 100

    print("laber_seeds")
    print(label_seeds)
    number_seeds = seeds.getNumberOfSuperpixels()
    mask_inv_seeds = cv2.bitwise_not(mask_seeds)

    print(np.array(mask_inv_seeds).shape)

    plt.hist(mask_inv_seeds)
    plt.show()

    img_seeds = cv2.bitwise_and(img, img, mask=mask_inv_seeds)

    return img_seeds

def compute_SLIC_skimage(path):
    img = imageio.imread(path)
    segments = slic(img, n_segments=60, compactness=10)
    out = mark_boundaries(img, segments)
    return out


def use_SLIC():
    path1 = "../out/-x0.png"
    path2 = "../out/+x0.png"

    img_slic_1 = compute_SEEDS_opencv(path1)
    # img_slic_2 = compute_SEEDS_opencv(path2)

    # plt.figure()
    # plt.subplot(121)
    # plt.imshow(img_slic_1)
    # plt.subplot(122)
    # plt.imshow(img_slic_2)
    # plt.show()

def rgb_to_face(rgb):
    return rgb[0]*255*255+rgb[1]*255+rgb[2]

def mayavi_model(vertices, faces, cell_data_custom):
    # cell_data_custom = [0.1 for i in range(99328)]
    mesh = mlab.triangular_mesh(vertices[:, 0], vertices[:, 1], vertices[:, 2],
                                faces)
    cell_data = mesh.mlab_source.dataset.cell_data
    cell_data.scalars = cell_data_custom
    cell_data.scalars.name = "cell data"
    cell_data.update()
    mesh2 = mlab.pipeline.set_active_attribute(mesh, cell_scalars='cell data')
    mlab.pipeline.surface(mesh2)
    mlab.show()
    return vertices, faces

def slic_saliency_to_obj():

    png_path = os.listdir("../out2")
    png_path_list = ["../out2/"+i for i in png_path]
    png_list = [cv2.imread(i) for i in png_path_list]


def mean_saliency_to_obj():
    vertices, faces = load_obj("../data/bunny.obj")
    # get image saliency
    path1 = "../out/-x0.png"
    path2 = "../out/-y0.png"
    # get index image
    index_path1 = "../out/-x1.png"
    index_path2 = "../out/-y1.png"

    ft1 = ft_saliency(path1)
    ft2 = ft_saliency(path2)
    ft_index1 = cv2.imread(index_path1)
    ft_index2 = cv2.imread(index_path2)

    img = [cv2.imread(path1), cv2.imread(path2)]
    lab = [ft1, ft2]
    index_img = [ft_index1, ft_index2]

    # each pixel saliency to triangle saliency
    # face saliency filled with ft saliency
    face_saliency = []
    for i in range(len(faces)):
        face_saliency.append([0])
    #
    len_faces = len(faces)
    img_shape = cv2.imread(path1).shape
    for i in range(img_shape[0]):
        for j in range(img_shape[1]):
            for p in range(len(img)):
                # for all the picture and its pixels
                # compute the color to index and save it
                pixel = index_img[p][i][j]
                face_index = rgb_to_face(pixel)
                if face_index > len_faces or face_index <= 0:
                    None
                else:
                    # cus i + 1 to rgb as face color
                    face_index -= 1
                    # face which out of list means background color, ignore it
                    face_saliency[face_index].append(lab[p][i][j])

    face_saliency_mean = []
    # traverse all the face and check its saliency
    for i in range(len(face_saliency)):
        saliency_list = np.array(face_saliency[i])
        face_saliency_mean.append(np.mean(saliency_list))

    # mayavi_model(vertices, faces, face_saliency_mean)
    print(len(face_saliency_mean))
    print(len(faces))
    print(face_saliency_mean[1])


use_SLIC()
