import cv2
import os

import numpy as np
from mayavi import mlab

from vispy import io
from tvtk.api import tvtk


def load_obj(filename):
    vertices_ = []
    faces_ = []
    img_textures_ = []
    face_textures_ = []

    for line in open(filename):
        if line.startswith('#'):
            continue
        values = line.split()
        if values[0] == 'v':
            v = [float(x) for x in values[1:4]]
            v = [v[0], v[1], v[2]]
            vertices_.append(v)
        elif values[0] == 'vt':
            img_textures_.append([values[1], values[2]])
        elif values[0] == 'f':
            f = [int(values[1].split('/')[0])-1,
                 int(values[2].split('/')[0])-1,
                 int(values[3].split('/')[0])-1]
            face_texture = [int(values[1].split('/')[1]) - 1,
                            int(values[2].split('/')[1]) - 1,
                            int(values[3].split('/')[1]) - 1]
            faces_.append(f)
            face_textures_.append(face_texture)
            
    vertices_ = np.array(vertices_)
    faces_ = np.array(faces_)
    img_textures_ = np.array(img_textures_)
    face_textures_ = np.array(face_textures_)
    return vertices_, faces_, img_textures_, face_textures_

# vec_list, index_list = test_intersection_plane()
# test_intersection_triangle(vec_list, index_list)
def use_mayavi():
    vertices, faces, normals, nothing = io.read_mesh("data/arma-modi.obj")

    start = np.array([0, 0, 0])
    end = np.array([0.1, 0.1, 0.1])
    direction = np.array(end) - start
    mlab.points3d(vertices[:, 0], vertices[:, 1], vertices[:, 2])
    mlab.plot3d([start[0], end[0]], [start[1], end[1]], [start[2], end[2]])
    mlab.show()

    # know that function is wrong
def mayavi_show_model():
    vertices, faces, normals, nothing = io.read_mesh("data/arma-modi.obj")

    start = np.array([0, 0, 0])
    end = np.array([0.1, 0.1, 0.1])

    mlab.triangular_mesh(vertices[:, 0], vertices[:, 1], vertices[:, 2], faces, colormap="bone")
    mlab.plot3d([start[0], end[0]], [start[1], end[1]], [start[2], end[2]])
    mlab.show()

def custom_colormap():
    x, y = np.mgrid[-10:10:200j, -10:10:200j]
    z = 100 * np.sin(x * y) / (x * y)

    # Visualize it with mlab.surf
    mlab.figure(bgcolor=(1, 1, 1))
    surf = mlab.surf(z, colormap='cool')

    # Retrieve the LUT of the surf object.
    lut = surf.module_manager.scalar_lut_manager.lut.table.to_array()

    # The lut is a 255x4 array, with the columns representing RGBA
    # (red, green, blue, alpha) coded with integers going from 0 to 255.

    # We modify the alpha channel to add a transparency gradient
    lut[:, -1] = np.linspace(0, 255, 256)
    # and finally we put this LUT back in the surface object. We could have
    # added any 255*4 array rather than modifying an existing LUT.
    surf.module_manager.scalar_lut_manager.lut.table = lut

    # We need to force update of the figure now that we have changed the LUT.
    mlab.draw()
    mlab.view(10, 85)

    mlab.show()

def face_color_triangular():
    n = 8
    t = np.linspace(-np.pi, np.pi, n)
    z = np.exp(1j * t)
    x = z.real.copy()
    y = z.imag.copy()
    z = np.zeros_like(x)

    triangles = [(0, i, i + 1) for i in range(n)]
    x = np.r_[0, x]
    y = np.r_[0, y]
    z = np.r_[1, z]
    t = np.r_[0, t]

    # These are the scalar values for each triangle
    # [1,2,3] = t
    # t[[2,1,0]] = [3, 2, 1]
    f = np.mean(t[np.array(triangles)], axis=1)

    # Plot it
    mesh = mlab.triangular_mesh(x, y, z, triangles)
    cell_data = mesh.mlab_source.dataset.cell_data
    cell_data.scalars = f
    cell_data.scalars.name = 'Cell data'
    cell_data.update()

    mesh2 = mlab.pipeline.set_active_attribute(mesh,
                                               cell_scalars='Cell data')
    mlab.pipeline.surface(mesh2)

    mlab.show()

def mayavi_cube():
    A = np.array([1, 1, 1])  # 0
    B = np.array([1, 1, -1])  # 1
    C = np.array([-1, 1, -1])  # 2
    D = np.array([-1, 1, 1])  # 3

    E = np.array([-1, -1, 1])  # 4
    F = np.array([-1, -1, -1])  # 5
    G = np.array([1, -1, 1])  # 6
    H = np.array([1, -1, -1])  # 7
    vertices = np.array([A, B, C, D, E, F, G, H])
    faces = np.array([
        # ACB, ACD, AED, AEG, AHB, AHG
        [0, 2, 1], [0, 2, 3], [0, 4, 3], [0, 4, 6], [0, 7, 1], [0, 7, 6],
        # FDC, FDE, FBC, FBH, FGE, FGH
        [5, 3, 2], [5, 3, 4], [5, 1, 2], [5, 1, 7], [5, 6, 4], [5, 6, 7]])
    mesh = mlab.triangular_mesh(vertices[:, 0], vertices[:, 1], vertices[:, 2],
                                faces)
    cell_data = mesh.mlab_source.dataset.cell_data
    cell_data.scalars = np.array([0.08*i for i in range(12)])
    cell_data.scalars.name = "cell data"
    cell_data.update()
    mesh2 = mlab.pipeline.set_active_attribute(mesh, cell_scalars='cell data')
    mlab.pipeline.surface(mesh2)
    mlab.show()
    return vertices, faces

def show_bunny_with_texture():
    obj_path = "data/bunny.obj"
    texture_path = "data/bunny-atlas.jpg"
    vertices, faces, img_textures, face_textures = load_obj(obj_path)

    vertex_uv = np.zeros(len(vertices)) - 1
    for face in range(len(faces)):
        three_point = faces[face]
        for i in range(3):
            vertex_uv[three_point[i]] = face_textures[face][i]

    texture = cv2.imread(texture_path)
    vertices_uv = np.zeros(len(vertices))
    for i in range(len(vertex_uv)):
        u, v = img_textures[i]
        u, v = np.float(u), np.float(v)
        m, n = int(u * 1024), int(v * 1024)
        r, g, b = texture[m, n]
        gray = 0.299 * r + 0.587 * g + 0.114 * b
        vertices_uv[i] = gray

    mesh = mlab.triangular_mesh(vertices[:, 0], vertices[:, 1], vertices[:, 2], faces)
    point_data = mesh.mlab_source.dataset.point_data
    point_data.scalars = vertices_uv
    print(len(vertices))
    print(len(vertices_uv))
    print(vertices_uv[:10])
    point_data.scalars.name = "point data"
    point_data.update()
    mesh2 = mlab.pipeline.set_active_attribute(mesh, point_scalars="point data")
    mlab.pipeline.surface(mesh2)
    mlab.show()

def mayavi_demo():
    # Create the data.
    from numpy import pi, sin, cos, mgrid
    dphi, dtheta = pi/250.0, pi/250.0
    [phi,theta] = mgrid[0:pi+dphi*1.5:dphi,0:2*pi+dtheta*1.5:dtheta]
    m0 = 4; m1 = 3; m2 = 2; m3 = 3; m4 = 6; m5 = 2; m6 = 6; m7 = 4;
    r = sin(m0*phi)**m1 + cos(m2*phi)**m3 + sin(m4*theta)**m5 + cos(m6*theta)**m7
    x = r*sin(phi)*cos(theta)
    y = r*cos(phi)
    z = r*sin(phi)*sin(theta)

    # View it.
    from mayavi import mlab
    s = mlab.mesh(x, y, z)
    mlab.show()

show_bunny_with_texture()
# face_color_triangular()
# mayavi_show_model()
# mayavi_demo()
