import cv2
import matplotlib.pyplot as plt
import numpy as np
import os
import itertools
import glob
import argparse
from PIL import Image
import csv
from scipy.ndimage.measurements import label
from skimage import exposure
from skimage import img_as_float
from skimage.transform import resize
from numpy import average, dot, linalg
import os

from adjustText import adjust_text

EPSILON = np.finfo('float').eps

def mean(l):
    tmp = 0.0
    for i in l:
        tmp += i
    return tmp/len(l)

def NSS(saliency_map, fixation_map):
	s_map = np.array(saliency_map, copy=False)
	f_map = np.array(fixation_map, copy=False) > 0.5
	if s_map.shape != f_map.shape:
		s_map = resize(s_map, f_map.shape)
	# Normalize saliency map to have zero mean and unit std
	s_map = normalize(s_map, method='standard')
	# Mean saliency value at fixation locations
	return np.mean(s_map[f_map])

# def main():
#     parameters = {}
#
#     parser = argparse.ArgumentParser()
#     parser.add_argument('-t', '--testList', nargs='+')
#     parser.add_argument('-r', '--referenceList', nargs='+')
#
#     # parser.print_help()
#     args = parser.parse_args()
#
#     for key, value in parser.parse_args()._get_kwargs():
#         if value is not None:
#             parameters[key] = value
#     # parameters = {'testList': ['lovis2018-3feature_3/*'],
#     #               'referenceList': ['locvis_2018-marking(328same)/*']}
#     parameters = {'testList': [r'D:\Dr_file\JND-plan\images\result\papers result\locvis_2018_papers(tiaozheng)\locvis_2018_flip\*'],
#                   'referenceList': ['./locvis_2018-marking(328tiaozheng)/*']}
#     # D:\Dr_file\JND-plan\images\locvis_2018-marking(286change)
#     # D:\Dr_file\JND-plan\images\locvis_2018-marking(328change)
#     # parameters = { 'testList': ['untitled.png'], 'referenceList': ['Attic_Aliasing_l1_aliasing-marking.png']}
#     # print(parameters)
#     run(parameters)


# def run(parameters):
#     refList = [glob.glob(el) for el in parameters['referenceList']]
#     refList = list(itertools.chain(*refList))
#
#     testList = [glob.glob(el) for el in parameters['testList']]
#     testList = list(itertools.chain(*testList))
#
#
#     if (len(testList) == 0 or len(refList) == 0):
#         print("Number of Tests:", len(testList))
#         print("Number of References:", len(refList))
#         print("One or both lists contain zero elements.")
#     elif (len(testList) == len(refList)):
#         for l1, l2 in zip(testList, refList):
#             print(l1, "  <----->  ", l2)
#         multiplePrediction(testList, refList, parameters)
#     else:
#         print("Different Number of elements. Test and Reference should be paired.")
#         print("Number of Tests:", len(testList))
#         print("Number of References:", len(refList))


# def multiplePrediction(testList, refList, parameters):
#     for test, ref in zip(testList, refList):
#         predict(test, ref)


# def predict(test, ref):
#     testFilename = test
#     referenceFileName = ref
#
#     basename, extension = os.path.splitext(os.path.basename(testFilename))
#
#     print("Distorted: ", testFilename, " Reference: ", referenceFileName)
    # print("Prediction...")
    # Reading images and allocating space for result image
    # refImage = cv2.imread(referenceFileName)
    # dstImage = cv2.imread(testFilename)
# refImage = cv2.imread('gd_truth_front_bunny-map.png')
# dstImage = cv2.imread('method_median_ac.png')
# refImage = cv2.cvtColor(refImage, cv2.COLOR_BGR2GRAY)
# dstImage = cv2.cvtColor(dstImage, cv2.COLOR_BGR2GRAY)
#
#     # image1 = Image.open(referenceFileName)
#     # image2 = Image.open(testFilename)
# image1 = Image.open('gd_truth_front_bunny-map.png')
# image2 = Image.open('method_median_ac.png')
#
# # origWidth, origHeight, channels = refImage.shape
# ss = calculate_ssim(refImage, dstImage)
# ps = PSNR(refImage, dstImage)
# cc = CC(refImage, dstImage)
# kld = KLD(refImage, dstImage)
# nss = NSS(refImage, dstImage)
#
# cosin = image_similarity_vectors_via_numpy(image1, image2)
# image1 = make_regalur_image(image1)
# image2 = make_regalur_image(image2)
# hist = calc_similar(image1, image2)
#
# print(ss)
# print(ps)
# print(cc)
# print(kld)
# print(nss)
# print(cosin)
# print(hist)

    # file = open("locvis_2018_flip.csv", "a+", newline="")
    # csv_writer = csv.writer(file)
    # # csv_writer.writerow([referenceFileName,testFilename,ss,ps])
    # csv_writer.writerow([testFilename, ss, ps, cc, kld, nss, cosin, hist])


def ssim(img1, img2):
    C1 = (0.01 * 255) ** 2
    C2 = (0.03 * 255) ** 2

    img1 = img1.astype(np.float64)
    img2 = img2.astype(np.float64)
    kernel = cv2.getGaussianKernel(11, 1.5)
    window = np.outer(kernel, kernel.transpose())

    mu1 = cv2.filter2D(img1, -1, window)[5:-5, 5:-5]  # valid
    mu2 = cv2.filter2D(img2, -1, window)[5:-5, 5:-5]
    mu1_sq = mu1 ** 2
    mu2_sq = mu2 ** 2
    mu1_mu2 = mu1 * mu2
    sigma1_sq = cv2.filter2D(img1 ** 2, -1, window)[5:-5, 5:-5] - mu1_sq
    sigma2_sq = cv2.filter2D(img2 ** 2, -1, window)[5:-5, 5:-5] - mu2_sq
    sigma12 = cv2.filter2D(img1 * img2, -1, window)[5:-5, 5:-5] - mu1_mu2

    ssim_map = ((2 * mu1_mu2 + C1) * (2 * sigma12 + C2)) / ((mu1_sq + mu2_sq + C1) *
                                                            (sigma1_sq + sigma2_sq + C2))
    return ssim_map.mean()


def PSNR(img1, img2, shave_border=0):
    height, width = img1.shape[:2]
    img1 = img1[shave_border:height - shave_border, shave_border:width - shave_border]
    img2 = img2[shave_border:height - shave_border, shave_border:width - shave_border]
    imdff = img1 - img2
    rmse = np.math.sqrt(np.mean(imdff ** 2))
    if rmse == 0:
        return 100
    return 20 * np.math.log10(255.0 / rmse)


def calculate_ssim(img1, img2):
    '''calculate SSIM
    the same outputs as MATLAB's
    img1, img2: [0, 255]
    '''
    if not img1.shape == img2.shape:
        raise ValueError('Input images must have the same dimensions.')
    if img1.ndim == 2:
        return ssim(img1, img2)
    elif img1.ndim == 3:
        if img1.shape[2] == 3:
            ssims = []
            for i in range(3):
                ssims.append(ssim(img1, img2))
            return np.array(ssims).mean()
        elif img1.shape[2] == 1:
            return ssim(np.squeeze(img1), np.squeeze(img2))
    else:
        raise ValueError('Wrong input image dimensions.')


def CC(img1, img2):
    map1 = np.array(img1, copy=False)
    map2 = np.array(img2, copy=False)
    if map1.shape != map2.shape:
        map1 = resize(map1, map2.shape, order=3,
                      mode='constant')  # bi-cubic/nearest is what Matlab imresize() does by default
    # Normalize the two maps to have zero mean and unit std
    map1 = normalize(map1, method='standard')
    map2 = normalize(map2, method='standard')
    # Compute correlation coefficient
    return np.corrcoef(map1.ravel(), map2.ravel())[0, 1]


def KLD(img1, img2):
    p = normalize(img1, method='sum')
    q = normalize(img2, method='sum')
    return np.sum(np.where(p != 0, p * np.log((p + EPSILON) / (q + EPSILON)), 0))


def NSS(img1, img2):
    s_map = np.array(img1, copy=False)
    f_map = np.array(img2, copy=False) > 0.5
    if s_map.shape != f_map.shape:
        s_map = resize(s_map, f_map.shape)
    # Normalize saliency map to have zero mean and unit std
    s_map = normalize(s_map, method='standard')
    # Mean saliency value at fixation locations
    return np.mean(s_map[f_map])


def normalize(x, method='standard', axis=None):
    np.seterr(divide='ignore', invalid='ignore')
    x = np.array(x, copy=False)
    if axis is not None:
        y = np.rollaxis(x, axis).reshape([x.shape[axis], -1])
        shape = np.ones(len(x.shape))
        shape[axis] = x.shape[axis]
        if method == 'standard':
            res = (x - np.mean(y, axis=1).reshape(shape)) / np.std(y, axis=1).reshape(shape)
        elif method == 'range':
            res = (x - np.min(y, axis=1).reshape(shape)) / (np.max(y, axis=1) - np.min(y, axis=1)).reshape(shape)
        elif method == 'sum':
            res = x / np.float_(np.sum(y, axis=1).reshape(shape))
        else:
            raise ValueError('method not in {"standard", "range", "sum"}')
    else:
        if method == 'standard':
            res = (x - np.mean(x)) / np.std(x)
        elif method == 'range':
            res = (x - np.min(x)) / (np.max(x) - np.min(x))
        elif method == 'sum':
            res = x / float(np.sum(x))
        else:
            raise ValueError('method not in {"standard", "range", "sum"}')
    return res

def make_regalur_image(img, size=(64, 64)):
    gray_image = img.resize(size).convert('RGB')
    return gray_image


# 计算直方图
def hist_similar(lh, rh):
    assert len(lh) == len(rh)
    hist = sum(1 - (0 if l == r else float(abs(l - r)) / max(l, r)) for l, r in zip(lh, rh)) / len(lh)
    return hist


# 计算直方图相似度
def calc_similar(li, ri):
    calc_sim = hist_similar(li.histogram(), ri.histogram())
    return calc_sim


# 对图片进行统一化处理
def get_thum(image, size=(64, 64), greyscale=False):
    # 利用image对图像大小重新设置, Image.ANTIALIAS为高质量的
    image = image.resize(size, Image.ANTIALIAS)
    if greyscale:
        # 将图片转换为L模式，其为灰度图，其每个像素用8个bit表示
        image = image.convert('L')
    return image


# 计算图片的余弦距离
def image_similarity_vectors_via_numpy(image1, image2):
    image1 = get_thum(image1)
    image2 = get_thum(image2)
    images = [image1, image2]
    vectors = []
    norms = []
    for image in images:
        vector = []
        for pixel_tuple in image.getdata():
            vector.append(average(pixel_tuple))
        vectors.append(vector)
        # linalg=linear（线性）+algebra（代数），norm则表示范数
        # 求图片的范数
        norms.append(linalg.norm(vector, 2))
    a, b = vectors
    a_norm, b_norm = norms
    # dot返回的是点积，对二维数组（矩阵）进行计算
    res = dot(a / a_norm, b / b_norm)
    return res

def SIM(saliency_map1, saliency_map2):
    map1 = np.array(saliency_map1, copy=False)
    map2 = np.array(saliency_map2, copy=False)
    if map1.shape != map2.shape:
        map1 = resize(map1, map2.shape, order=3, mode='constant') # bi-cubic/nearest is what Matlab imresize() does by default
    # Normalize the two maps to have values between [0,1] and sum up to 1
    map1 = normalize(map1, method='range')
    map2 = normalize(map2, method='range')
    map1 = normalize(map1, method='sum')
    map2 = normalize(map2, method='sum')
    # Compute histogram intersection
    intersection = np.minimum(map1, map2)
    return np.sum(intersection)

def to_black(img):
    h, w, _ = img.shape
    for x in range(h):
        for y in range(w):
            pixel = img[x][y]
            if pixel[0] == 255 and pixel[1] == 255 and pixel[2] == 255:
                img[x][y] = [0, 0, 0]
    return img

def two_float(num):
    return int(num*100)/100

def draw_fig_bar():

    # path = "/home/simula/Papers/mypaper/Double Columns/figs/dvsm_different_methods_bar.png"
    # img = cv2.imread(path)
    # img = np.array(img)
    # print(img.shape)
    # img = img[130:1265, 250:2320, :]
    # cv2.imwrite("/home/simula/Papers/mypaper/Double Columns/figs/dvsm_different_methods_bar_c.png", img)
    # plt.imshow(img)
    # plt.show()
    # exit()

    # main()
    # 9->0 8->180 7->90 6->270

    res = []
    g = 1
    for i in ["ac", "deep", "ft", "gbvs", "lc", "salicon"]:
        plt.subplot(2, 3, g)
        g += 1
        # father_path = "/home/simula/Pic/paper_bunny/aug/" + i + "/"
        father_path = "E:/mycodes/Dataset/pic/paper_bunny/aug/" + i + "/"
        paths = ["0", "90", "180", "270"]
        gd = [6, 8, 7, 9]

        cc_list, sim_list, kld_list, nss_list  = [], [], [], []
        for j in range(len(paths)):
            dst_path = father_path + paths[j] + ".png"
            ref_path = "E:/mycodes/Dataset/pic/paper_experiment/fig/" + str(gd[j]) + "_2.png"

            dst_img = cv2.imread(dst_path)
            ref_img = cv2.imread(ref_path)
            dst_img = cv2.resize(dst_img, (512, 512))

            cc = CC(ref_img, dst_img)
            sim = SIM(ref_img, dst_img)
            kld = KLD(ref_img, dst_img)
            nss = NSS(ref_img, dst_img)

            cc = two_float(cc)
            sim = two_float(sim)
            kld = two_float(kld)
            nss = two_float(nss)

            cc_list.append(cc)
            sim_list.append(sim)
            kld_list.append(kld)
            nss_list.append(nss)

            # ss = calculate_ssim(refImage, dstImage)
            # ps = PSNR(refImage, dstImage)
            # nss = NSS(refImage, dstImage)

            print("method = {}, view = {}".format(i, paths[j]))
            print("cc = {}, sim = {}, kld = {}, nss = {}".format(cc, sim, kld, nss))

            # plt.subplot(121)
            # plt.imshow(ref_img)
            # plt.subplot(122)
            # plt.imshow(dst_img)
            # plt.show()

        tick_label = ["+X", "+Y", "-X", "-Y"]
        k = np.arange(len(cc_list))
        bar_width = 0.25
        font_size = 8

        plt.xlabel(i.upper())
        plt.ylabel("Metrics")
        plt.bar(k, cc_list, bar_width)
        plt.bar(k+bar_width, sim_list, bar_width)
        plt.bar(k+bar_width*2, kld_list, bar_width)
        plt.bar(k+bar_width*3, nss_list, bar_width)

        plt.ylim([0, 5])
        plt.xticks(k+bar_width/2, tick_label)

        for a, b in zip(k, cc_list):
            plt.text(a, b, b, ha='center', va='bottom', fontsize=font_size)
        for a, b in zip(k+bar_width, sim_list):
            plt.text(a, b, b, ha='center', va='bottom', fontsize=font_size)
        for a, b in zip(k+bar_width*2, kld_list):
            plt.text(a, b, b, ha='center', va='bottom', fontsize=font_size)
        for a, b in zip(k+bar_width*3, nss_list):
            plt.text(a, b, b, ha='center', va='bottom', fontsize=font_size)

        print("==> {} cc list mean is {}".format(i, mean(cc_list)))
        print("==> {} sim list mean is {}".format(i, mean(sim_list)))
        print("==> {} kld list mean is {}".format(i, mean(kld_list)))
        print("==> {} nss list mean is {}".format(i, mean(nss_list)))

    plt.show()


def cut_number(num):
    return int(num*100)/100

def draw_fig_plot():
    # 最后的图的形式为

    paths = ["0", "90", "180", "270"]
    gd = [6, 8, 7, 9]
    g = 1
    x = ["+X", "+Y", "-X", "-Y"]
    ac, deep, ft, gbvs, lc, salicon = [], [], [], [], [], []
    ac2, deep2, ft2, gbvs2, lc2, salicon2 = [], [], [], [], [], []
    ac3, deep3, ft3, gbvs3, lc3, salicon3 = [], [], [], [], [], []
    ac4, deep4, ft4, gbvs4, lc4, salicon4 = [], [], [], [], [], []

    for j in range(len(paths)):

        for i in ["ac", "deep", "ft", "gbvs", "lc", "salicon"]:
            father_path = "/home/simula/Pic/paper_bunny/aug/" + i + "/"
            father_path = "E:/mycodes/Dataset/pic/paper_bunny/aug/" + i + "/"

            dst_path = father_path + paths[j] + ".png"
            ref_path = "/home/simula/Pic/paper_experiment/fig/" + str(gd[j]) + "_2.png"
            ref_path = "E:/mycodes/Dataset/pic/paper_experiment/fig/" + str(gd[j]) + "_2.png"
            
            dst_img = cv2.imread(dst_path)
            ref_img = cv2.imread(ref_path)
            dst_img = cv2.resize(dst_img, (512, 512))

            cc = CC(ref_img, dst_img)
            sim = SIM(ref_img, dst_img)
            kld = KLD(ref_img, dst_img)
            nss = NSS(ref_img, dst_img)

            if i == "ac":
                ac.append(cc)
                ac2.append(sim)
                ac3.append(kld)
                ac4.append(nss)

            if i == "deep":
                deep.append(cc)
                deep2.append(sim)
                deep3.append(kld)
                deep4.append(nss)

            if i == "ft":
                ft.append(cc)
                ft2.append(sim)
                ft3.append(kld)
                ft4.append(nss)

            if i == "gbvs":
                gbvs.append(cc)
                gbvs2.append(sim)
                gbvs3.append(kld)
                gbvs4.append(nss)

            if i == "lc":
                lc.append(cc)
                lc2.append(sim)
                lc3.append(kld)
                lc4.append(nss)

            if i == "salicon":
                salicon.append(cc)
                salicon2.append(sim)
                salicon3.append(kld)
                salicon4.append(nss)


    ac = [cut_number(i) for i in ac]
    deep = [cut_number(i) for i in deep]
    ft = [cut_number(i) for i in ft]
    gbvs = [cut_number(i) for i in gbvs]
    lc = [cut_number(i) for i in lc]
    salicon = [cut_number(i) for i in salicon]

    ac2 = [cut_number(i) for i in ac2]
    deep2 = [cut_number(i) for i in deep2]
    ft2 = [cut_number(i) for i in ft2]
    gbvs2 = [cut_number(i) for i in gbvs2]
    lc2 = [cut_number(i) for i in lc2]
    salicon2 = [cut_number(i) for i in salicon2]

    ac3 = [cut_number(i) for i in ac3]
    deep3 = [cut_number(i) for i in deep3]
    ft3 = [cut_number(i) for i in ft3]
    gbvs3 = [cut_number(i) for i in gbvs3]
    lc3 = [cut_number(i) for i in lc3]
    salicon3 = [cut_number(i) for i in salicon3]

    ac4 = [cut_number(i) for i in ac4]
    deep4 = [cut_number(i) for i in deep4]
    ft4 = [cut_number(i) for i in ft4]
    gbvs4 = [cut_number(i) for i in gbvs4]
    lc4 = [cut_number(i) for i in lc4]
    salicon4 = [cut_number(i) for i in salicon4]

    print("----")
    print(ac)
    print(deep)
    print(ft)
    print(gbvs)
    print(lc)
    print(salicon)

    print("----")
    print(ac2)
    print(deep2)
    print(ft2)
    print(gbvs2)
    print(lc2)
    print(salicon2)

    print("----")
    print(ac3)
    print(deep3)
    print(ft3)
    print(gbvs3)
    print(lc3)
    print(salicon3)

    print("---")
    print(mean(ac3))
    print(mean(deep3))
    print(mean(ft3))
    print(mean(gbvs3))
    print(mean(lc3))
    print(mean(salicon3))

    exit(0)

    # -------------------------------------------------------------------------------------- #
    # -------------------------------------------------------------------------------------- #
    # -------------------------------------------------------------------------------------- #

    fontsize = 15

    #--#

    plt.subplot(1, 3, 1)
    plt.plot(x, ac, label="AC", marker=".")
    plt.plot(x, deep, label="Deep", marker="v")
    plt.plot(x, ft, label="FT", marker="o")
    plt.plot(x, gbvs, label="GBVS", marker=",")
    plt.plot(x, lc, label="LC", marker="^")
    plt.plot(x, salicon, label="Salicon", marker="s")

    
    for a, b in zip(x, ac):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, deep):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, ft):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, gbvs):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, lc):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, salicon):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    
    plt.xlabel("View", fontsize=fontsize)
    plt.ylabel("Evaluation", fontsize=fontsize)

    plt.xticks(fontsize=fontsize)
    plt.yticks(fontsize=fontsize)

    plt.legend()
    plt.title("Use CC to assess")

# -------------------------------------------------------------------------------------- #
# -------------------------------------------------------------------------------------- #
# -------------------------------------------------------------------------------------- #

    plt.subplot(1, 3, 2)
    plt.plot(x, ac2, label="AC", marker=".")
    plt.plot(x, deep2, label="Deep", marker="v")
    plt.plot(x, ft2, label="FT", marker="o")
    plt.plot(x, gbvs2, label="GBVS", marker=",")
    plt.plot(x, lc2, label="LC", marker="^")
    plt.plot(x, salicon2, label="Salicon", marker="s")
    

    for a, b in zip(x, ac2):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, deep2):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, ft2):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, gbvs2):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, lc2):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, salicon2):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)

    plt.xlabel("View", fontsize=fontsize)
    plt.ylabel("Evaluation", fontsize=fontsize)

    plt.xticks(fontsize=fontsize)
    plt.yticks(fontsize=fontsize)

    plt.legend()
    plt.title("Use SIM to assess")


# -------------------------------------------------------------------------------------- #
# -------------------------------------------------------------------------------------- #
# -------------------------------------------------------------------------------------- #
#
    ax = plt.subplot(1, 3, 3)

    plt.plot(x, ac3, label="AC", marker=".")
    plt.plot(x, deep3, label="Deep", marker="v")
    plt.plot(x, ft3, label="FT", marker="o")
    plt.plot(x, gbvs3, label="GBVS", marker=",")
    plt.plot(x, lc3, label="LC", marker="^")
    plt.plot(x, salicon3, label="Salicon", marker="s")

    for a, b in zip(x, ac3):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, deep3):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, ft3):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, gbvs3):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, lc3):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)
    for a, b in zip(x, salicon3):
        plt.text(a, b, b, ha='center', va='bottom', fontsize=fontsize)


    plt.xlabel("View", fontsize=fontsize)
    plt.ylabel("Evaluation", fontsize=fontsize)

    plt.xticks(fontsize=fontsize)
    plt.yticks(fontsize=fontsize)

    plt.legend()
    plt.title("Use KLD to assess")


    plt.show()

draw_fig_plot()
# draw_fig_bar()



