# 问题描述和解决方法：
# 1. 对于投影，主要还是利用模型外接立方体的扩展立方体来进行投影，
# 2. 投影的方式为原点到目标像素的空间位置构成的射线，然后求其和模型的相交
# 3. 根据相交的面片得知面片的位置，进而得到面片的颜色和像素的颜色
# 4. 最终得到立方体投影其中一个面的颜色

# 实验过程分解
# 1. 模型外接立方体，立方体扩展
# 2. 模型分割
# 3. 遍历每个像素
# 4. 可以从外向内射出射线吗？这样可以使用AABB盒进行加速

import math
from matplotlib import path
from os.path import split
# import trimesh
from vispy import scene, io
import cv2 as cv
import numpy as np
from matplotlib import pyplot as plt
import random
from mayavi import mlab

def load_obj(filename):
    vertices_ = []
    faces_ = []
    for line in open(filename):
        if line.startswith('#'):
            continue
        values = line.split()
        if values[0] == 'v':
            v = [float(x) for x in values[1:4]]
            v = (v[0], v[1], v[2])
            vertices_.append(v)
        elif values[0] == 'f':
            f = (int(values[1].split('/')[0])-1,
                 int(values[2].split('/')[0])-1,
                 int(values[3].split('/')[0])-1)
            faces_.append(f)
    vertices_ = np.array(vertices_)
    faces_ = np.array(faces_)
    return vertices_, faces_

def norm(v):
    length = len(v)
    end = 0
    for i in range(length):
        end += v[i] * v[i]
    return np.sqrt(end)


def show_line(ax, p1, p2):
    ax.plot([p1[0], p2[0]], [p1[1], p2[1]], [p1[2], p2[2]])
    return ax


def show_triangle(axes, p1, p2, p3, color='r'):
    axes.plot([p1[0], p2[0]], [p1[1], p2[1]], [p1[2], p2[2]], c=color)
    axes.plot([p1[0], p3[0]], [p1[1], p3[1]], [p1[2], p3[2]], c=color)
    axes.plot([p3[0], p2[0]], [p3[1], p2[1]], [p3[2], p2[2]], c=color)
    return axes


def coordiante_change(vertices):
    x = vertices[:, 0]
    y = vertices[:, 1]
    z = vertices[:, 2]
    x_min, x_max = min(x), max(x)
    y_min, y_max = min(y), max(y)
    z_min, z_max = min(z), max(z)
    point_min = np.array([x_min, y_min, z_min])
    point_max = np.array([x_max, y_max, z_max])
    direction = point_max - point_min
    t = 0.5
    for index in range(len(vertices)):
        vertex = np.array(vertices[index])
        new_vertex = vertex - t * direction
        vertices[index] = new_vertex
    return vertices


def out_cube(vertices):
    x = vertices[:, 0]
    y = vertices[:, 1]
    z = vertices[:, 2]
    x_min, x_max = min(x), max(x)
    y_min, y_max = min(y), max(y)
    z_min, z_max = min(z), max(z)
    return max(x_max - x_min, y_max - y_min, z_max - z_min)


def cube_c(c):
    image_cube_c = 2 * c
    # front
    front = np.array([1, 0, 0])
    cube_front = c * front
    # behind
    behind = np.array([-1, 0, 0])
    cube_behind = c * behind
    # left
    left = np.array([0, -1, 0])
    cube_left = c * left
    # right
    right = np.array([0, 1, 0])
    cube_right = c * right
    # up
    up = np.array([0, 0, 1])
    cube_up = c * up
    # bottom
    bottom = np.array([0, 0, -1])
    cube_bottom = c * bottom

    return


def build_map_vermesh(vertices, faces):
    # 这里的keys是从0开始的faces的index
    v_to_mesh = dict()
    for index in range(len(faces)):
        face = faces[index]
        for vertex in face:
            if vertex not in v_to_mesh.keys():
                v_to_mesh[vertex] = [index]
            else:
                v_to_mesh[vertex].append(index)
    return v_to_mesh


def compute_pixel_to_space(c, tag):
    A_ = np.array([c, c, c])  # 0
    B_ = np.array([c, c, -c])  # 1
    C_ = np.array([-c, c, -c])  # 2
    D_ = np.array([-c, c, c])  # 3

    E_ = np.array([-c, -c, c])  # 4
    F_ = np.array([-c, -c, -c])  # 5
    G_ = np.array([c, -c, c])  # 6
    H_ = np.array([c, -c, -c])  # 7

    if tag == 1:
        A, B, D = A_, B_, D_
    elif tag == 2:
        A, B, D = D_, C_, E_
    elif tag == 3:
        A, B, D = E_, F_, G_
    elif tag == 4:
        A, B, D = G_, H_, A_
    elif tag == 5:
        A, B, D = D_, A_, E_
    elif tag == 6:
        A, B, D = C_, B_, F_

    # default watch the model from the top
    O = np.array([0, 0, 0])
    HEIGHT = 400
    WIDTH = 400
    _OA = A - O
    _AB = B - A
    _AD = D - A
    _AB_n = 1 / norm(_AB) * _AB
    _AD_n = 1 / norm(_AD) * _AD
    dist = 2 * c / HEIGHT
    pixel_in_space = np.zeros(HEIGHT * HEIGHT * 3).reshape([HEIGHT, HEIGHT, 3])
    for i in range(HEIGHT):
        for j in range(HEIGHT):
            pixel_in_space[i][j] = _OA + dist * i * _AB_n + dist * j * _AD_n
    return pixel_in_space


# return the tag of the vertices
def split_vertex(vertex):
    x = vertex[0]
    y = vertex[1]
    z = vertex[2]
    if x > 0 and y > 0 and z > 0:
        return 1
    elif x > 0 and y > 0 and z < 0:
        return 2
    elif x > 0 and y < 0 and z > 0:
        return 3
    elif x > 0 and y < 0 and z < 0:
        return 4

    elif x < 0 and y > 0 and z > 0:
        return 5
    elif x < 0 and y < 0 and z < 0:
        return 6
    elif x < 0 and y < 0 and z > 0:
        return 7
    else:
        return 8

def split_model_v1(vertices):
    slice_abcd = []
    for index in range(len(vertices)):
        vertex = vertices[index]
        y = vertex[1]
        if y > 0:
            slice_abcd.append(index)
    return slice_abcd

# visual all the split and judge the correction.
def split_model_v2(vertices, v_to_mesh):
    c = 2

    abcd_vertices = []
    height, width = 100, 100
    dist = 2*c/100
    A = np.array([c, c, c])  # 0
    B = np.array([c, c, -c])  # 1
    C = np.array([-c, c, -c])  # 2
    start = np.array([0, 0, 0])
    slice_abcd = []
    for i in range(height*width):
        slice_abcd.append([])

    for index in range(len(vertices)):
        vertex = vertices[index]
        vertex = np.array(vertex)
        vec, t = intersection_plane(start, vertex-start,
                                    A, B, C)
        x, y, z = vec[0], vec[1], vec[2]
        if -c < x < c and -c < z < c and y == c:
            t_i = int((c - x)/dist)
            t_j = int((c - z)/dist)
            slice_abcd[t_j*100+t_i].append(index)

    return slice_abcd, abcd_vertices  # slice_abcd each item contains the related points


def intersection_plane(start, direction, p1, p2, p3):
    p1 = np.array(p1)
    p2 = np.array(p2)
    p3 = np.array(p3)
    direction = np.array(direction) / norm(direction)
    # three point build a plane, and compute the intersection which in the plane
    # need to judge the direction of the light,
    plane_norm = np.cross(p2 - p1, p3 - p1)
    t = np.dot((p1 - start), plane_norm) / np.dot(plane_norm, direction)
    # then judge the point is in the triangle or not
    vec = start + t * direction
    return vec, t


def intersection_triangle(end, p1, p2, p3):
    s_total = norm(np.cross(p1 - p2, p1 - p3))
    s1 = norm(np.cross(end - p1, end - p2))
    s2 = norm(np.cross(end - p1, end - p3))
    s3 = norm(np.cross(end - p3, end - p2))
    diff = abs(s1+s2+s3-s_total)
    if diff < 1e-6:
        return True
    return False
    # return math.isclose(s1 + s2 + s3, s_total)


def face_to_rgb(face):
    rgb = []
    c = face
    for i in range(3):
        t = c % 255
        c = int(c / 255)
        rgb.append(t)
    rgb.reverse()
    return rgb


def generate_cube():
    A = np.array([1, 1, 1])  # 0
    B = np.array([1, 1, -1])  # 1
    C = np.array([-1, 1, -1])  # 2
    D = np.array([-1, 1, 1])  # 3

    E = np.array([-1, -1, 1])  # 4
    F = np.array([-1, -1, -1])  # 5
    G = np.array([1, -1, 1])  # 6
    H = np.array([1, -1, -1])  # 7
    vertices = np.array([A, B, C, D, E, F, G, H])
    faces = np.array([
        # ACB, ACD, AED, AEG, AHB, AHG
        [0, 2, 1], [0, 2, 3], [0, 4, 3], [0, 4, 6], [0, 7, 1], [0, 7, 6],
        # FDC, FDE, FBC, FBH, FGE, FGH
        [5, 3, 2], [5, 3, 4], [5, 1, 2], [5, 1, 7], [5, 6, 4], [5, 6, 7]])
    mlab.triangular_mesh(vertices[:, 0], vertices[:, 1], vertices[:, 2], faces)
    mlab.show()
    return vertices, faces


def judge_split_model():
    vertices, faces, normals, nothing = io.read_mesh("../data/arma-modi.obj")
    vertices = coordiante_change(vertices)
    slice_abcd, vec_abcd = split_model_v2(vertices, faces)
    vec_abcd = np.array(vec_abcd)
    print("vec len = {}".format(len(vec_abcd)))
    max_length = 0
    for i in slice_abcd:
        if len(i) > max_length:
            max_length = len(i)
    print("max_length len = {}".format(max_length))

    mlab.triangular_mesh(vertices[:, 0], vertices[:, 1], vertices[:, 2], faces)
    mlab.points3d(vec_abcd[:, 0], vec_abcd[:, 1], vec_abcd[:, 2])
    mlab.show()


def main():
    index = 6
    face_direction = np.array([
        [0, 0, 0],
        [0, -1, 0], [1, 0, 0], [0, 1, 0], [-1, 0, 0], [0, 0, -1], [0, 0, 1]
    ])
    # read the obj file
    # vertices, faces, normals, nothing = io.read_mesh("../data/arma-modi.obj")
    vertices, faces = load_obj("../data/venus.obj")
    vertices = coordiante_change(vertices)  # change the basic coordinate
    pixel_in_space = compute_pixel_to_space(2, index)
    start1 = pixel_in_space[0][399]
    start2 = pixel_in_space[0][0]
    start3 = pixel_in_space[399][0]
    start4 = pixel_in_space[399][399]
    start5 = pixel_in_space[200][200]

    direct = face_direction[index]
    end1 = start1 + 4*direct
    end2 = start2 + 4*direct
    end3 = start3 + 4*direct
    end4 = start4 + 4*direct
    end5 = start5 + 4*direct

    print(start1, end1)
    print(start2, end2)
    print(start3, end3)
    print(start4, end4)
    print(start5, end5)

    # for index in range(len(faces)):
    #     three_point = faces[index]
    #     p1, p2, p3 = vertices[three_point[0]], vertices[three_point[1]], vertices[three_point[2]]
    #     vec, t = intersection_plane(start5, direct,
    #                                 p1, p2, p3)
    #     # judge the new point in the plane or not
    #     if intersection_triangle(vec,
    #                              p1, p2, p3):
    #         print("index {} intersection".format(index))
    mlab.triangular_mesh(vertices[:, 0], vertices[:, 1], vertices[:, 2], faces)
    mlab.plot3d([start1[0], end1[0]], [start1[1], end1[1]], [start1[2], end1[2]])
    mlab.plot3d([start2[0], end2[0]], [start2[1], end2[1]], [start2[2], end2[2]])
    mlab.plot3d([start3[0], end3[0]], [start3[1], end3[1]], [start3[2], end3[2]])
    mlab.plot3d([start4[0], end4[0]], [start4[1], end4[1]], [start4[2], end4[2]])
    mlab.plot3d([start5[0], end5[0]], [start5[1], end5[1]], [start5[2], end5[2]])

    # mlab.plot3d([start[0], 1], [start[1], 1], [start[2], 1])
    mlab.show()


# judge_split_model()
main()
# point_plane()
# test_intersection_triangle()
# generate_cube()
# mayavi_cube()
# mayavi_show_model()

# test()
# visual()

# search_bug()
# the problem is on the split of the ij, and the judge direction is
# change the split_model_v2() and util the accurate ij_index

